/*
Kinect-In'tear'active Curtain for Processing 2.x

::DESTRUCTIONS::
Recommend on load hitting 'r' as occasionally the Kinect routine
tears the curtain on initialisation. (ie: it detects a random
blob below it's depth threshold)

-- Kinect bits --
d = Show / Hide Depth Image, current threshold & stats (FPS)
up = increase depth threshold
down = decrease depth threshold
q = tilt up
a = tilt down

To adjust the minimum radius below threshold required to interact
with Fabric, adjust 'int minRad = 500;' line in KinectTracker tab.

-- Fabric bits --
g = gravity on / off
r = reset fabric
t = toggle between Grid and Chains

-- Syphon --
Syphon Instance: 'Fabric'

-- OSC --
Broadcasts on port 7000 by default... change on line 56.
/fabric/handpos {x,y} -- raw hand x/y
/fabric/lerppos {x,y} -- lerped hand x/y (use with handpos for velocity calcs)
/fabric/gravity {int} -- current gravity value
/fabric/grid    {1/0} -- 0 = grid, 1 = chains
/fabric/thresh  {int} -- depth threshold

Listens on port 5001 by default... change on line 57.

-- Required Libraries --
Processing 2 or 3
Syphon
Open Kinect for Processing (NOT SimpleOpenNI)
OSCP5

-- Credits --
Original Curtain/SuperFabric (Fabric Simulator) by BlueThen: July 19th, 2011
www.bluethen.com
  
Kinect AveragePointTracking and Syphon Output added by Marci: Nov 7th, 2015
*/

class Particle {
  PVector lastPosition;
  PVector position;
   
  PVector acceleration;
   
  float mass = 1;
  float damping = 20;
 
  ArrayList links = new ArrayList();
   
  boolean pinned = false;
  PVector pinLocation = new PVector(0,0);
   
  // Particle constructor
  Particle (PVector pos) {
    position = pos.get();
    lastPosition = pos.get();
    acceleration = new PVector(0,0);
  }
   
  void updatePhysics (float timeStep) {
    
    PVector fg = new PVector(0, mass * gravity);
    this.applyForce(fg);
    
    PVector velocity = PVector.sub(position, lastPosition);
    acceleration.sub(PVector.mult(velocity,damping/mass));
    PVector nextPos = PVector.add(PVector.add(position, velocity), PVector.mult(PVector.mult(acceleration, 0.5), timeStep * timeStep));
     
    lastPosition.set(position);
    position.set(nextPos);
    acceleration.set(0,0,0);
  }
  void updateInteractions () {
    if (tracker.found==1) {
      float distanceSquared = distPointToSegmentSquared(v2.x,v2.y,v1.x,v1.y,position.x,position.y);
        if (distanceSquared < mouseInfluenceSize) {
          lastPosition = PVector.sub(position, new PVector((v1.x-v2.x)*mouseInfluenceScalar, (v1.y-v2.y)*mouseInfluenceScalar));
        }
    }
  }
 
  void draw () {
    stroke(255);
    if (links.size() > 0) {
      for (int i = 0; i < links.size(); i++) {
        Link currentLink = (Link) links.get(i);
        currentLink.draw();
      }
    }
    else{
      point(position.x, position.y);
    }
    
  }
  
  void solveConstraints () {
    for (int i = 0; i < links.size(); i++) {
      Link currentLink = (Link) links.get(i);
      currentLink.constraintSolve();
    }

    if (position.y < 1)
      position.y = 2 * (1) - position.y;
    if (position.y > height-1)
      position.y = 2 * (height - 1) - position.y;
    if (position.x > width-1)
      position.x = 2 * (width - 1) - position.x;
    if (position.x < 1)
      position.x = 2 * (1) - position.x;
     
    if (pinned)
      position.set(pinLocation);
  }
   
  void attachTo (Particle P, float restingDist, float stiff) {
    Link lnk = new Link(this, P, restingDist, stiff);
    links.add(lnk);
  }
  void removeLink (Link lnk) {
    links.remove(lnk);
  } 
  
  void applyForce (PVector f) {
    acceleration.add(PVector.div(f, mass));
  }
   
  void pinTo (PVector location) {
    pinned = true;
    pinLocation.set(location);
  }
}

class Link {
  float restingDistance;
  float stiffness;
   
  Particle p1;
  Particle p2;
   
  float scalarP1;
  float scalarP2;
  
  boolean drawThis = true;
   
  Link (Particle which1, Particle which2, float restingDist, float stiff) {
    p1 = which1; 
    p2 = which2; 
     
    restingDistance = restingDist;
    stiffness = stiff;
    
    float im1 = 1 / p1.mass; // inverse mass quantities
    float im2 = 1 / p2.mass;
    scalarP1 = (im1 / (im1 + im2)) * stiffness;
    scalarP2 = (im2 / (im1 + im2)) * stiffness;
  }
   
  void constraintSolve () {
    PVector delta = PVector.sub(p1.position, p2.position); 
    float d = sqrt(delta.x * delta.x + delta.y * delta.y);
    float difference = (restingDistance - d) / d;
     
    if (d > curtainTearSensitivity)
      p1.removeLink(this);
      p1.position.add(PVector.mult(delta, scalarP1 * difference));
      p2.position.sub(PVector.mult(delta, scalarP2 * difference));
  }
 
  void draw () {
    if (drawThis)
      line(p1.position.x, p1.position.y, p2.position.x, p2.position.y);
  }
}
 
ArrayList particles;
 
// every particle within this many pixels will be influenced by the depth-point
float mouseInfluenceSize = 15;

float mouseTearSize = 8;
float mouseInfluenceScalar = 1;
 
// force of gravity is really 9.8, but because of scaling, we use 9.8 * 40 (392)
// (9.8 is too small for a 1 second timestep)
float gravity = 392;
 
// Dimensions for our curtain. These are number of particles for each direction, not actual widths and heights
// the true width and height can be calculated by multiplying restingDistances by the curtain dimensions
final int curtainHeight = 47;
final int curtainWidth = 63;
final int yStart = 0; // where will the curtain start on the y axis?
final float restingDistances = 10;
final float stiffnesses = 1;
final float curtainTearSensitivity = 80; // distance the particles have to go before ripping
 
long previousTime;
long currentTime;
final int fixedDeltaTime = 15;
float fixedDeltaTimeSeconds = (float)fixedDeltaTime / 1000.0;
int leftOverDeltaTime = 0;
int constraintAccuracy = 3;

int ShowDepth=0;
int toggleGrH=0;

void setup5() {
  
  mouseInfluenceSize *= mouseInfluenceSize;
  mouseTearSize *= mouseTearSize;
   
  createCurtain();  
}
 
void draw5() {
  background(0);
  fill(30, 30, 30, 200);
  noStroke();
  ellipse(v1.x, v1.y, 20, 20);
  fill(20, 20, 20, 200);
  noStroke();
  ellipse(v2.x, v2.y, 20, 20);
  
  int t = tracker.getThreshold();
  if (ShowDepth == 1){
    tracker.display();
  }
  
  /******** Physics ********/
  currentTime = millis();
  long deltaTimeMS = currentTime - previousTime;
  previousTime = currentTime; // reset previousTime
  
  int timeStepAmt = (int)((float)(deltaTimeMS + leftOverDeltaTime) / (float)fixedDeltaTime);
  timeStepAmt = min(timeStepAmt, 5);
   
  leftOverDeltaTime += (int)deltaTimeMS - (timeStepAmt * fixedDeltaTime); 
  
  // This keeps the program from exploding from user interaction if the timeStepAmt gets too high.
  mouseInfluenceScalar = 1.0 / timeStepAmt;
   
  // update physics
  for (int iteration = 1; iteration <= timeStepAmt; iteration++) {
    for (int x = 0; x < constraintAccuracy; x++) {
      for (int i = 0; i < particles.size(); i++) {
        Particle particle = (Particle) particles.get(i);
        particle.solveConstraints();
      }
    }
    
    for (int i = 0; i < particles.size(); i++) {
      Particle particle = (Particle) particles.get(i);
      particle.updateInteractions();
      particle.updatePhysics(fixedDeltaTimeSeconds);
    }
  }
  // draw each particle or its links
  for (int i = 0; i < particles.size(); i++) {
    Particle particle = (Particle) particles.get(i);
    particle.draw();
  }
  
  if (ShowDepth==1){
    if (frameCount % 60 == 0){
      println("Depth threshold: " + t + "    " +  "framerate: " + int(frameRate));
      //println(osc.tcpServer().getClients().length);
    }
    fill(255,0,0,255);
    text("maxthreshold: " + t + " minthreshold: " + mt + "   " +  "framerate: " + int(frameRate) + "    " + 
    "UP increase threshold, DOWN decrease threshold", 10, 500);
  }
  server.sendScreen();
  
}
void createCurtain () {
  particles = new ArrayList();
   
  int midWidth = (int) (width/2 - (curtainWidth * restingDistances)/2);
  for (int y = 0; y <= curtainHeight; y++) {
    for (int x = 0; x <= curtainWidth; x++) {
      Particle particle = new Particle(new PVector(midWidth + x * restingDistances, y * restingDistances + yStart));
       
      if(toggleGrH == 0){
        if (x != 0)
          particle.attachTo((Particle)(particles.get(particles.size()-1)), restingDistances, stiffnesses);
      }
      
      if (y != 0)
        particle.attachTo((Particle)(particles.get((y - 1) * (curtainWidth+1) + x)), restingDistances, stiffnesses);
         
      // we pin the very top particles to where they are
      if (y == 0)
        particle.pinTo(particle.position);
         
      // add to particle array 
      particles.add(particle);
    }
  }
}
 

void toggleGravity () {
  if (gravity != 0){
    gravity = 0;
  } else {
    gravity = 192;
  }
  OscMessage myMsg4 = new OscMessage("/fabric/gravity");
  myMsg4.add(new Object[] {gravity});
  osc.send(myMsg4, izzyloop);
}

void toggleDepth() {
  if(ShowDepth != 0)
    ShowDepth = 0;
  else
    ShowDepth = 1;
}

void toggleGrid() {
  if(toggleGrH != 0){
    toggleGrH = 0;
  } else {
    toggleGrH = 1;
  }
  OscMessage myMsg3 = new OscMessage("/fabric/grid");
  myMsg3.add(new Object[] {toggleGrH});
  osc.send(myMsg3, izzyloop);
}
 
// Credit to: http://www.codeguru.com/forum/showpost.php?p=1913101&postcount=16
float distPointToSegmentSquared (float lineX1, float lineY1, float lineX2, float lineY2, float pointX, float pointY) {
  float vx = lineX1 - pointX;
  float vy = lineY1 - pointY;
  float ux = lineX2 - lineX1;
  float uy = lineY2 - lineY1;
   
  float len = ux*ux + uy*uy;
  float det = (-vx * ux) + (-vy * uy);
  if ((det < 0) || (det > len)) {
    ux = lineX2 - pointX;
    uy = lineY2 - pointY;
    return min(vx*vx+vy*vy, ux*ux+uy*uy);
  }
   
  det = ux*vy - uy*vx;
  return (det*det) / len;
}
